@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:sw=4:et
 """
 Base class for UML model elements.
 """
@@ -12,6 +11,7 @@ from event import ElementDeleteEvent
 from gaphor.misc import uniqueid
 from properties import umlproperty, association
 
+
 class Element(object):
     """
     Base class for UML data classes.
@@ -33,14 +33,16 @@ class Element(object):
         self._id = id or (id is not False and uniqueid.generate_id() or False)
         # The factory this element belongs to.
         self._factory = factory
-        self._observers = dict()
         self.__in_unlink = mutex.mutex()
 
+
     id = property(lambda self: self._id, doc='Id')
 
+
     factory = property(lambda self: self._factory,
                        doc="The factory that created this element")
 
+
     def umlproperties(self):
         """
         Iterate over all UML properties 
@@ -53,6 +55,7 @@ class Element(object):
                 if isinstance(prop, umlprop):
                     yield prop
 
+
     def save(self, save_func):
         """
         Save the state by calling save_func(name, value).
@@ -60,6 +63,7 @@ class Element(object):
         for prop in self.umlproperties():
             prop.save(self, save_func)
 
+
     def load(self, name, value):
         """
         Loads value in name. Make sure that for every load postload()
@@ -73,6 +77,7 @@ class Element(object):
         else:
             prop.load(self, value)
 
+
     def postload(self):
         """
         Fix up the odds and ends.
@@ -80,6 +85,7 @@ class Element(object):
         for prop in self.umlproperties():
             prop.postload(self)
 
+
     def unlink(self):
         """
         Unlink the element. All the elements references are destroyed.
@@ -102,6 +108,7 @@ class Element(object):
         """
         return isinstance(self, class_)
 
+
     def isTypeOf(self, other):
         """
         Returns true if the object is of the same type as other.
@@ -109,6 +116,20 @@ class Element(object):
         return type(self) == type(other)
 
 
+    def __getstate__(self):
+        d = dict(self.__dict__)
+        try:
+            del d['_factory']
+        except KeyError:
+            pass
+        return d
+
+
+    def __setstate__(self, state):
+        self._factory = None
+        self.__dict__.update(state)
+
+
 try:
     import psyco
 except ImportError:
@@ -116,3 +137,4 @@ except ImportError:
 else:
     psyco.bind(Element)
 
+# vim:sw=4:et
