@@ -41,7 +41,8 @@ import operator
 class umlproperty(object):
     """
     Superclass for attribute, enumeration and association.
-    The subclasses should define a 'name' attribute that contains the name
+
+    The subclasses should define a ``name`` attribute that contains the name
     of the property. Derived properties (derivedunion and redefine) can be
     connected, they will be notified when the value changes.
     """
@@ -73,16 +74,6 @@ class umlproperty(object):
         """
         pass
 
-    def notify(self, obj):
-        """
-        Notify obj that the property's value has been changed.
-        Deriviates are also triggered to send a notify signal.
-        """
-        try:
-            obj.notify(self.name, pspec=self)
-        except Exception, e:
-            log.error(str(e), e)
-
 
 class attribute(umlproperty):
     """
@@ -134,7 +125,6 @@ class attribute(umlproperty):
         else:
             setattr(obj, self._name, value)
         component.handle(AttributeChangeEvent(obj, self, old, value))
-        self.notify(obj)
 
     def _del(self, obj, value=None):
         old = self._get(obj)
@@ -145,13 +135,16 @@ class attribute(umlproperty):
             pass
         else:
             component.handle(AttributeChangeEvent(obj, self, old, self.default))
-            self.notify(obj)
 
 
 class enumeration(umlproperty):
     """
-    Enumeration.
-    Element.enum = enumeration('enum', ('one', 'two', 'three'), 'one')
+    Enumeration
+
+      Element.enum = enumeration('enum', ('one', 'two', 'three'), 'one')
+
+    An enumeration is a special kind of attribute that can only hold a
+    predefined set of values. Multiplicity is always `[0..1]`
     """
 
     def __init__(self, name, values, default):
@@ -186,7 +179,6 @@ class enumeration(umlproperty):
         else:
             setattr(obj, self._name, value)
         component.handle(AttributeChangeEvent(obj, self, old, value))
-        self.notify(obj)
 
     def _del(self, obj, value=None):
         old = self._get(obj)
@@ -196,7 +188,6 @@ class enumeration(umlproperty):
             pass
         else:
             component.handle(AttributeChangeEvent(obj, self, old, self.default))
-            self.notify(obj)
 
 
 class association(umlproperty):
@@ -221,6 +212,7 @@ class association(umlproperty):
         self.upper = upper
         self.composite = composite
         self.opposite = opposite and intern(opposite)
+        self.stub = None
 
     def load(self, obj, value):
         if not isinstance(value, self.type):
@@ -293,7 +285,6 @@ class association(umlproperty):
 
             if value is None:
                 if do_notify:
-                    self.notify(obj)
                     component.handle(event)
                 return
 
@@ -314,15 +305,22 @@ class association(umlproperty):
 
         # Callbacks are only connected if a new relationship has
         # been established.
-        value.connect('__unlink__', self.__on_unlink, obj)
+#        value.connect('__unlink__', self.__on_unlink, obj)
 #        if self.composite:
 #            obj.connect('__unlink__', self.__on_composite_unlink, value)
         
         if not from_opposite and self.opposite:
-            getattr(type(value), self.opposite)._set(value, obj, from_opposite=True, do_notify=do_notify)
+            opposite = getattr(type(value), self.opposite)
+            if not opposite.opposite:
+                opposite.stub = self
+            opposite._set(value, obj, from_opposite=True, do_notify=do_notify)
+        elif not self.opposite:
+            if not self.stub:
+                self.stub = associationstub(self)
+                setattr(self.type, 'UML_associationstub_%x' % id(self), self.stub)
+            self.stub._set(value, obj)
 
         if do_notify:
-            self.notify(obj)
             component.handle(event)
 
     def _del(self, obj, value, from_opposite=False, do_notify=True):
@@ -341,6 +339,9 @@ class association(umlproperty):
 
         if not from_opposite and self.opposite:
             getattr(type(value), self.opposite)._del(value, obj, from_opposite=True)
+        elif not self.opposite:
+            if self.stub:
+                self.stub._del(value, obj, from_opposite=True)
 
         event = None
         if self.upper > 1:
@@ -367,11 +368,10 @@ class association(umlproperty):
                 if do_notify:
                     event = AssociationSetEvent(obj, self, value, None)
 
-        value.disconnect(self.__on_unlink, obj)
+#        value.disconnect(self.__on_unlink, obj)
 #        if self.composite:
 #            obj.disconnect(self.__on_composite_unlink, value)
         if do_notify and event:
-            self.notify(obj)
             component.handle(event)
 
     def unlink(self, obj):
@@ -383,21 +383,61 @@ class association(umlproperty):
 
             for value in list(values):
                 # TODO: make normal unlinking work through this method.
-                #self.__delete__(obj, value)
+                self.__delete__(obj, value)
                 if composite:
                     value.unlink()
 
-    def __on_unlink(self, value, pspec, obj):
-        """
-        Disconnect when the element on the other end of the association
-        (value) sends the '__unlink__' signal. This is especially important
-        for uni-directional associations.
-        """
-        #print '__on_unlink', name, obj, value
-        if pspec == '__unlink__':
-            self.__delete__(obj, value)
-            # re-establish unlink handler:
-            value.connect('__unlink__', self.__on_unlink, obj)
+
+class AssociationStubError(Exception):
+    pass
+
+
+class associationstub(umlproperty):
+    """
+    An association stub is an internal thingy that ensures all associations
+    are always bi-directional. This helps the application when one end of
+    the association is unlinked. On unlink() of an element all `umlproperty`'s
+    are iterated and called by their unlink() method.
+    """
+
+    def __init__(self, association):
+        self.association = association
+        self._name = intern('_stub_%x' % id(self))
+
+    def __get__(self, obj, class_=None):
+        if obj:
+            raise AssociationStubError, 'getting values not allowed'
+        return self
+
+    def __set__(self, obj, value):
+        raise AssociationStubError, 'setting values not allowed'
+
+    def __delete__(self, obj, value=None):
+        raise AssociationStubError, 'deleting values not allowed'
+
+    def save(self, obj, save_func):
+        pass
+
+    def load(self, obj, value):
+        pass
+
+    def unlink(self, obj):
+        try:
+            values = getattr(obj, self._name)
+        except AttributeError:
+            pass
+        else:
+            for value in set(values):
+                self.association.__delete__(value, obj)
+
+    def _set(self, obj, value):
+        try:
+            getattr(obj, self._name).add(value)
+        except AttributeError:
+            setattr(obj, self._name, set([value]))
+            
+    def _del(self, obj, value, from_opposite=False):
+        getattr(obj, self._name).discard(value)
 
 
 class derivedunion(umlproperty):
@@ -522,18 +562,20 @@ class derivedunion(umlproperty):
                     if values:
                         new_value = iter(values).next()
                     component.handle(DerivedUnionSetEvent(event.element, self, old_value, new_value))
-            self.notify(event.element)
 
 
 class redefine(umlproperty):
     """
     Redefined association
-    Element.x = redefine('x', Class, Element.assoc)
+
+      Element.redefine = redefine(Element, 'redefine', Class, Element.assoc)
+
     If the redefine eclipses the original property (it has the same name)
     it ensures that the original values are saved and restored.
     """
 
-    def __init__(self, name, type, original):
+    def __init__(self, decl_class, name, type, original):
+        self.decl_class = decl_class
         self.name = intern(name)
         self._name = intern('_' + name)
         self.type = type
@@ -592,7 +634,7 @@ class redefine(umlproperty):
 
     @component.adapter(IAssociationChangeEvent)
     def _association_changed(self, event):
-        if event.property is self.original:
+        if event.property is self.original and isinstance(event.element, self.decl_class):
             # mimic the events for Set/Add/Delete
             if IAssociationSetEvent.providedBy(event):
                 component.handle(RedefineSetEvent(event.element, self, event.old_value, event.new_value))
@@ -602,7 +644,6 @@ class redefine(umlproperty):
                 component.handle(RedefineDeleteEvent(event.element, self, event.old_value))
             else:
                 log.error('Don''t know how to handle event ' + str(event) + ' for redefined association')
-            self.notify(event.element)
 
 
 try:
