@@ -1,8 +1,11 @@
 #!/usr/bin/env python
 
 import unittest
+from zope import component
+from gaphor.application import Application
 from gaphor.UML.properties import *
 from gaphor.UML.element import Element
+from gaphor.UML.event import AssociationChangeEvent
 
 class PropertiesTestCase(unittest.TestCase):
 
@@ -70,14 +73,14 @@ class PropertiesTestCase(unittest.TestCase):
 
         assert a.one is b
         assert b.two is a
-        assert len(a._observers.get('__unlink__')) == 1
-        assert len(b._observers.get('__unlink__')) == 1
+        #assert len(a._observers.get('__unlink__')) == 0
+        #assert len(b._observers.get('__unlink__')) == 0
 
         a.one = B()
         assert a.one is not b
         assert b.two is None
-        assert len(a._observers.get('__unlink__')) == 1
-        assert len(b._observers.get('__unlink__')) == 0
+        #assert len(a._observers.get('__unlink__')) == 0
+        #assert len(b._observers.get('__unlink__')) == 0
 
         c = C()
         try:
@@ -90,8 +93,8 @@ class PropertiesTestCase(unittest.TestCase):
         del a.one
         assert a.one is None
         assert b.two is None
-        assert len(a._observers.get('__unlink__')) == 0
-        assert len(b._observers.get('__unlink__')) == 0
+        #assert len(a._observers.get('__unlink__')) == 0
+        #assert len(b._observers.get('__unlink__')) == 0
 
 
     def test_association_1_n(self):
@@ -119,8 +122,8 @@ class PropertiesTestCase(unittest.TestCase):
         assert len(b1.two) == 1, 'len(b1.two) == %d' % len(b1.two)
         assert a1 in b1.two
         assert a1.one is b1, '%s/%s' % (a1.one, b1)
-        assert len(a1._observers.get('__unlink__')) == 1
-        assert len(b1._observers.get('__unlink__')) == 1
+        #assert len(a1._observers.get('__unlink__')) == 0
+        #assert len(b1._observers.get('__unlink__')) == 0
 
         b1.two = a2
         assert a1 in b1.two
@@ -147,8 +150,8 @@ class PropertiesTestCase(unittest.TestCase):
         assert a2 in b1.two
         assert a1.one is None
         assert a2.one is b1
-        assert len(a1._observers.get('__unlink__')) == 0
-        assert len(b1._observers.get('__unlink__')) == 1
+        #assert len(a1._observers.get('__unlink__')) == 0
+        #assert len(b1._observers.get('__unlink__')) == 0
 
         a2.one = b2
 
@@ -194,8 +197,8 @@ class PropertiesTestCase(unittest.TestCase):
         assert a1 in b1.two
         assert a1 in b2.two
         assert not a2.one
-        assert len(a1._observers.get('__unlink__')) == 2
-        assert len(b1._observers.get('__unlink__')) == 1
+        #assert len(a1._observers.get('__unlink__')) == 0
+        #assert len(b1._observers.get('__unlink__')) == 0
 
         a2.one = b1
         assert len(a1.one) == 2
@@ -220,10 +223,80 @@ class PropertiesTestCase(unittest.TestCase):
         assert a1 in b2.two
         assert b1 in a2.one
         assert a2 in b1.two
-        assert len(a1._observers.get('__unlink__')) == 1
-        assert len(b1._observers.get('__unlink__')) == 1
+        #assert len(a1._observers.get('__unlink__')) == 0
+        #assert len(b1._observers.get('__unlink__')) == 0
 
-    def test_association_unlink(self):
+    def test_association_swap(self):
+        class A(Element): pass
+        class B(Element): pass
+        class C(Element): pass
+
+        A.one = association('one', B, 0, '*')
+
+        a = A()
+        b1 = B()
+        b2 = B()
+
+        a.one = b1
+        a.one = b2
+        assert a.one.size() == 2
+        assert a.one[0] is b1
+        assert a.one[1] is b2
+
+        events = []
+        @component.adapter(AssociationChangeEvent)
+        def handler(event, events=events):
+            events.append(event)
+
+        Application.register_handler(handler)
+        try:
+            a.one.swap(b1, b2)
+            assert len(events) == 1
+            assert events[0].property is A.one
+            assert events[0].element is a
+        finally:
+            Application.unregister_handler(handler)
+
+        assert a.one.size() == 2
+        assert a.one[0] is b2
+        assert a.one[1] is b1
+
+    def test_association_unlink_1(self):
+        #
+        # unlink
+        #
+        class A(Element): pass
+        class B(Element): pass
+        class C(Element): pass
+
+        A.one = association('one', B, 0, '*')
+
+        a1 = A()
+        a2 = A()
+        b1 = B()
+        b2 = B()
+
+        a1.one = b1
+        a1.one = b2
+        assert b1 in a1.one
+        assert b2 in a1.one
+
+        a2.one = b1
+        #assert len(a1._observers.get('__unlink__')) == 0
+        #assert len(b1._observers.get('__unlink__')) == 0
+
+        # remove b1 from all elements connected to b1
+        # also the signal should be removed
+        b1.unlink()
+
+        #assert len(a1._observers.get('__unlink__')) == 1, a1._observers.get('__unlink__')
+        #assert len(b1._observers.get('__unlink__')) == 0, b1._observers.get('__unlink__')
+
+        assert b1 not in a1.one
+        assert b2 in a1.one
+
+
+    def test_association_unlink_2(self):
         #
         # unlink
         #
@@ -247,14 +320,14 @@ class PropertiesTestCase(unittest.TestCase):
         assert a1 in b2.two
 
         a2.one = b1
-        assert len(a1._observers.get('__unlink__')) == 2
-        assert len(b1._observers.get('__unlink__')) == 2
+        #assert len(a1._observers.get('__unlink__')) == 0
+        #assert len(b1._observers.get('__unlink__')) == 0
 
         # remove b1 from all elements connected to b1
         # also the signal should be removed
         b1.unlink()
 
-        assert len(a1._observers.get('__unlink__')) == 1, a1._observers.get('__unlink__')
+        #assert len(a1._observers.get('__unlink__')) == 1, a1._observers.get('__unlink__')
         #assert len(b1._observers.get('__unlink__')) == 0, b1._observers.get('__unlink__')
 
         assert b1 not in a1.one
@@ -269,9 +342,9 @@ class PropertiesTestCase(unittest.TestCase):
         A.a = attribute('a', types.StringType, 'default')
 
         a = A()
-        assert a.a == 'default'
+        assert a.a == 'default', a.a
         a.a = 'bar'
-        assert a.a == 'bar'
+        assert a.a == 'bar', a.a
         del a.a
         assert a.a == 'default'
         try:
@@ -301,28 +374,28 @@ class PropertiesTestCase(unittest.TestCase):
         del a.a
         assert a.a == 'one'
 
-    def test_notify(self):
-        import types
-        class A(Element):
-            notified=None
-            def notify(self, name, pspec):
-                self.notified = name
-
-        A.assoc = association('assoc', A)
-        A.attr = attribute('attr', types.StringType, 'default')
-        A.enum = enumeration('enum', ('one', 'two'), 'one')
-
-        a = A()
-        assert a.notified == None
-        a.assoc = A()
-        assert a.notified == 'assoc', a.notified
-        a.attr = 'newval'
-        assert a.notified == 'attr', a.notified
-        a.enum = 'two'
-        assert a.notified == 'enum', a.notified
-        a.notified = None
-        a.enum = 'two' # should not notify since value hasn't changed.
-        assert a.notified == None
+#    def test_notify(self):
+#        import types
+#        class A(Element):
+#            notified=None
+#            def notify(self, name, pspec):
+#                self.notified = name
+#
+#        A.assoc = association('assoc', A)
+#        A.attr = attribute('attr', types.StringType, 'default')
+#        A.enum = enumeration('enum', ('one', 'two'), 'one')
+#
+#        a = A()
+#        assert a.notified == None
+#        a.assoc = A()
+#        assert a.notified == 'assoc', a.notified
+#        a.attr = 'newval'
+#        assert a.notified == 'attr', a.notified
+#        a.enum = 'two'
+#        assert a.notified == 'enum', a.notified
+#        a.notified = None
+#        a.enum = 'two' # should not notify since value hasn't changed.
+#        assert a.notified == None
 
     def test_derivedunion(self):
         class A(Element): pass
@@ -390,9 +463,6 @@ class PropertiesTestCase(unittest.TestCase):
         assert b.is_unlinked
 
     def test_derivedunion(self):
-        from zope import component
-        from gaphor.application import Application
-        from gaphor.UML.event import AssociationChangeEvent
         
         class A(Element):
             is_unlinked = False
@@ -537,7 +607,7 @@ class PropertiesTestCase(unittest.TestCase):
 
         A.a = association('a', A, upper=1)
 
-        A.a = redefine('a', A, A.a)
+        A.a = redefine(A, 'a', A, A.a)
         events = []
         @component.adapter(AssociationChangeEvent)
         def handler(event, events=events):
@@ -553,6 +623,48 @@ class PropertiesTestCase(unittest.TestCase):
         finally:
             Application.unregister_handler(handler)
 
+    def test_redefine_subclass(self):
+        from zope import component
+        from gaphor.application import Application
+        from gaphor.UML.event import AssociationChangeEvent
+        
+        class A(Element):
+            is_unlinked = False
+            def unlink(self):
+                self.is_unlinked = True
+                Element.unlink(self)
+
+        A.a = association('a', A, upper=1)
+
+        class B(A):
+            pass
+
+        B.b = redefine(B, 'b', A, A.a)
+        
+        events = []
+        @component.adapter(AssociationChangeEvent)
+        def handler(event, events=events):
+            events.append(event)
+
+        Application.register_handler(handler)
+        try:
+            a = A()
+            a.a = A()
+            # Only a.a changes, no B class involved
+            assert len(events) == 1
+            assert events[0].property is A.a, events[0].property
+            #assert events[1].property is A.a.original, events[1].property
+            del events[:]
+
+            a = B()
+            a.a = A()
+            # Now events are sent for both association and redefine
+            assert len(events) == 2
+            assert events[0].property is B.b, events[0].property
+            assert events[1].property is B.b.original, events[1].property
+        finally:
+            Application.unregister_handler(handler)
+
 
 if __name__ == '__main__':
     unittest.main()
