@@ -1,5 +1,8 @@
 """
-Adapters for the Property Editor
+Adapters for the Property Editor.
+
+To register property pages implemented in this module, it is imported in
+gaphor.adapter package.
 
 # TODO: make all labels align top-left
 # Add hidden columns for list stores where i can put the actual object
@@ -421,6 +424,8 @@ def watch_attribute(attribute, widget, handler):
     @component.adapter(IAttributeChangeEvent)
     def attribute_watcher(event):
         if attribute is None or event.property is attribute:
+            if widget.props.has_focus:
+                return
             handler(event)
 
     Application.register_handler(attribute_watcher)
@@ -488,6 +493,9 @@ class CommentItemPropertyPage(object):
         subject = self.context.subject
         page = gtk.VBox()
 
+        if not subject:
+            return page
+
         label = gtk.Label(_('Comment'))
         label.set_justify(gtk.JUSTIFY_LEFT)
         page.pack_start(label, expand=False)
@@ -499,6 +507,7 @@ class CommentItemPropertyPage(object):
         text_view.set_buffer(buffer)
         text_view.show()
         page.pack_start(text_view)
+        page.set_data('default', text_view)
 
         changed_id = buffer.connect('changed', self._on_body_change)
 
@@ -545,6 +554,7 @@ class NamedItemPropertyPage(object):
         entry = gtk.Entry()        
         entry.set_text(subject and subject.name or '')
         hbox.pack_start(entry)
+        page.set_data('default', entry)
 
         # monitor subject.name attribute
         changed_id = entry.connect('changed', self._on_name_change)
@@ -936,6 +946,9 @@ class AssociationPropertyPage(NamedItemPropertyPage):
         
     def construct_end(self, title, end):
         hbox = gtk.HBox()
+        if not end.subject:
+            return hbox
+
         label = gtk.Label(title)
         label.set_justify(gtk.JUSTIFY_LEFT)
         self.size_group.add_widget(label)
