@@ -11,7 +11,7 @@ All important services are present in the application object:
 
 import pkg_resources
 from zope import component
-from gaphor.interfaces import IService
+from gaphor.interfaces import IService, IEventFilter
 from gaphor.event import ServiceInitializedEvent, ServiceShutdownEvent
 import gaphor.UML
 
@@ -19,10 +19,10 @@ import gaphor.UML
 class _Application(object):
     """
     The Gaphor application is started from the Application instance. It behaves
-    as a singleton in many ways.
+    like a singleton in many ways.
 
     The Application is responsible for loading services and plugins. Services
-    are registered as "utilities" in a global registry.
+    are registered as "utilities" in the application registry.
 
     Methods are provided that wrap zope.component's handle, adapter and
     subscription registrations. In addition to registration methods also
@@ -34,7 +34,8 @@ class _Application(object):
     
     def __init__(self):
         self._uninitialized_services = {}
-        self.globalSiteManager = component.getGlobalSiteManager()
+        self.init_components()
+        self._event_filter = None
 
     def init(self, services=None):
         """
@@ -43,6 +44,32 @@ class _Application(object):
         self.load_services(services)
         self.init_all_services()
 
+    def init_components(self):
+        """
+        Initialize application level component registry.
+
+        Frequently used query methods are overridden on the zope.component
+        module.
+        """
+        #self._components = component.getGlobalSiteManager()
+        #return
+
+        self._components = component.registry.Components(name='app',
+                               bases=(component.getGlobalSiteManager(),))
+
+        # Make sure component.handle() and query methods works.
+        # TODO: eventually all queries should be done through the Application
+        # instance.
+        component.handle = self.handle
+        component.getMultiAdapter = self._components.getMultiAdapter
+        component.queryMultiAdapter = self._components.queryMultiAdapter
+        component.getAdapter = self._components.getAdapter
+        component.queryAdapter = self._components.queryAdapter
+        component.getAdapters = self._components.getAdapters
+        component.getUtility = self._components.getUtility
+        component.queryUtility = self._components.queryUtility
+        component.getUtilitiesFor = self._components.getUtilitiesFor
+
     def load_services(self, services=None):
         """
         Load services from resources.
@@ -51,7 +78,6 @@ class _Application(object):
         Service should provide an interface gaphor.interfaces.IService.
         """
         for ep in pkg_resources.iter_entry_points('gaphor.services'):
-            #print ep, dir(ep)
             log.debug('found entry point service.%s' % ep.name)
             cls = ep.load()
             if not IService.implementedBy(cls):
@@ -77,7 +103,7 @@ class _Application(object):
         else:
             log.info('initializing service service.%s' % name)
             srv.init(self)
-            self.globalSiteManager.registerUtility(srv, IService, name)
+            self._components.registerUtility(srv, IService, name)
             self.handle(ServiceInitializedEvent(name, srv))
             return srv
 
@@ -86,7 +112,7 @@ class _Application(object):
 
     def get_service(self, name):
         try:
-            return self.globalSiteManager.getUtility(IService, name)
+            return self._components.getUtility(IService, name)
         except component.ComponentLookupError:
             return self.init_service(name)
 
@@ -95,20 +121,15 @@ class _Application(object):
         gtk.main()
 
     def shutdown(self):
-        for name, srv in self.globalSiteManager.getUtilitiesFor(IService):
+        for name, srv in self._components.getUtilitiesFor(IService):
             srv.shutdown()
             self.handle(ServiceShutdownEvent(name, srv))
-            self.globalSiteManager.unregisterUtility(srv, IService, name)
+            self._components.unregisterUtility(srv, IService, name)
 
-        # Re-initialize Zope's global site manager
-        # (cleanup adapters and utilities):
-        try:
-            self.globalSiteManager.__init__('base')
-        except Exception, e:
-            log.error('Re-initialization of the Zope SiteManager failed', e)
-        self.__init__()
+        # Re-initialize components registry
+        self.init_components()
 
-    # Wrap zope.component's SiteManager methods
+    # Wrap zope.component's Components methods
 
     def register_adapter(self, factory, adapts=None, provides=None, name=''):
         """
@@ -116,7 +137,7 @@ class _Application(object):
         interface. A name can be used to distinguish between different adapters
         that adapt to the same interfaces.
         """
-        self.globalSiteManager.registerAdapter(factory, adapts, provides,
+        self._components.registerAdapter(factory, adapts, provides,
                               name, event=False)
 
     def unregister_adapter(self, factory=None,
@@ -124,14 +145,14 @@ class _Application(object):
         """
         Unregister a previously registered adapter.
         """
-        self.globalSiteManager.unregisterAdapter(factory,
+        self._components.unregisterAdapter(factory,
                               required, provided, name)
 
     def register_subscription_adapter(self, factory, adapts=None, provides=None):
         """
         Register a subscription adapter. See registerAdapter().
         """
-        self.globalSiteManager.registerSubscriptionAdapter(factory, adapts,
+        self._components.registerSubscriptionAdapter(factory, adapts,
                               provides, event=False)
 
     def unregister_subscription_adapter(self, factory=None,
@@ -139,30 +160,42 @@ class _Application(object):
         """
         Unregister a previously registered subscription adapter.
         """
-        self.globalSiteManager.unregisterSubscriptionAdapter(factory,
+        self._components.unregisterSubscriptionAdapter(factory,
                               required, provided, name)
 
     def register_handler(self, factory, adapts=None):
         """
         Register a handler. Handlers are triggered (executed) when specific
-        events are emited through the handle() method.
+        events are emitted through the handle() method.
         """
-        self.globalSiteManager.registerHandler(factory, adapts, event=False)
+        self._components.registerHandler(factory, adapts, event=False)
 
     def unregister_handler(self, factory=None, required=None):
         """
         Unregister a previously registered handler.
         """
-        self.globalSiteManager.unregisterHandler(factory, required)
+        self._components.unregisterHandler(factory, required)
  
-    def handle(self, *objects):
+    def _filter(self, objects):
+        filtered = list(objects)
+        for o in objects:
+            for adapter in self._components.subscribers(objects, IEventFilter):
+                if adapter.filter():
+                    # event is blocked
+                    filtered.remove(o)
+                    break
+        return filtered
+
+    def handle(self, *events):
         """
         Send event notifications to registered handlers.
         """
-        self.globalSiteManager.handle(*objects)
+        objects = self._filter(events)
+        if objects:
+            self._components.handle(*events)
+
 
 # Make sure there is only one!
 Application = _Application()
 
-
 # vim:sw=4:et:ai
