@@ -4,7 +4,7 @@ The Core module provides an entry point for Gaphor's core constructs.
 An average module should only need to import this module.
 """
 
-from gaphor.application import Application as _Application
+from gaphor.application import Application
 from gaphor.transaction import Transaction, transactional
 from gaphor.action import action, toggle_action, radio_action, build_action_group
 from gaphor.i18n import _
@@ -23,7 +23,7 @@ class inject(object):
     
     def __init__(self, name):
         self._name = name
-        self._inj = None
+        #self._s = None
         
     def __get__(self, obj, class_=None):
         """
@@ -31,9 +31,10 @@ class inject(object):
         """
         if not obj:
             return self
-        if self._inj is None:
-            self._inj = _Application.get_service(self._name)
-        return self._inj
+        return Application.get_service(self._name)
+        #if self._s is None:
+        #    self._s = _Application.get_service(self._name)
+        #return self._s
 
 
 # vim:sw=4:et:ai
