@@ -5,6 +5,7 @@ Basic functionality for canvas line based items on a diagram.
 from math import atan2, pi
 
 import gaphas
+from gaphor import UML
 from gaphas.util import text_extents, text_align
 from diagramitem import DiagramItem
 from interfaces import IConnect
@@ -13,6 +14,7 @@ from gaphor.diagram.style import get_text_point_at_line, \
     get_text_point_at_line2, \
     ALIGN_CENTER, ALIGN_LEFT, ALIGN_RIGHT, ALIGN_TOP
 
+
 class LineItem(gaphas.Line, DiagramItem):
     """
     Base class for diagram lines.
@@ -27,6 +29,16 @@ class LineItem(gaphas.Line, DiagramItem):
     tail = property(lambda self: self._handles[-1])
 
 
+    def setup_canvas(self):
+        gaphas.Line.setup_canvas(self)
+        self.register_handlers()
+
+
+    def teardown_canvas(self):
+        gaphas.Line.teardown_canvas(self)
+        self.unregister_handlers()
+
+
     def pre_update(self, context):
         # first, update stereotype to know its text
         self.update_stereotype()
@@ -174,7 +186,6 @@ class DiagramLine(LineItem):
 
 
 class NamedLine(DiagramLine):
-    __namedelement__ = True
 
     __style__ = {
             'name-align': (ALIGN_CENTER, ALIGN_TOP),
@@ -183,6 +194,22 @@ class NamedLine(DiagramLine):
             'name-align-str': None,
     }
 
+    def __init__(self, id=None):
+        DiagramLine.__init__(self, id)
+        self._name = self.add_text('name', style={
+                    'text-align': self.style.name_align,
+                    'text-padding': self.style.name_padding,
+                    'text-outside': self.style.name_outside,
+                    'text-align-str': self.style.name_align_str,
+                    'text-align-group': 'stereotype',
+                }, editable=True)
+        self.add_watch(UML.NamedElement.name, self.on_named_element_name)
+
+
+    def on_named_element_name(self, event):
+        if self.subject and (event is None or self.subject is event.element):
+            self._name.text = self.subject.name
+            self.request_update()
 
 
 class FreeLine(LineItem):
