@@ -3,12 +3,14 @@ Base classes related to items, which represent UML classes deriving
 from NamedElement.
 """
 
+from gaphor import UML
+from gaphor.UML.interfaces import IAttributeChangeEvent
+from gaphor.application import Application
 from gaphor.diagram.elementitem import ElementItem
 from gaphor.diagram.style import get_min_size, ALIGN_CENTER, ALIGN_TOP
 import gaphor.diagram.font as font
 
 class NamedItem(ElementItem):
-    __namedelement__ = True
 
     __style__ = {
         'min-size'    : (100, 50),
@@ -32,9 +34,18 @@ class NamedItem(ElementItem):
                 visible=self.is_namespace_info_visible,
                 font=font.FONT_SMALL)
 
+        self._name = self.add_text('name', style={
+                    'text-align': self.style.name_align,
+                    'text-padding': self.style.name_padding,
+                    'text-outside': self.style.name_outside,
+                    'text-align-str': self.style.name_align_str,
+                    'text-align-group': 'stereotype',
+                }, editable=True)
+
         # size of stereotype, namespace and name text
         self._header_size = 0, 0
-
+        self.add_watch(UML.NamedElement.name, self.on_named_element_name)
+        self.add_watch(UML.NamedElement.namespace, self.on_named_element_namespace)
 
     def is_namespace_info_visible(self):
         """
@@ -58,32 +69,26 @@ class NamedItem(ElementItem):
         return self._from.text and namespace is not canvas.diagram.namespace
 
 
-    def on_subject_notify(self, pspec, notifiers=()):
-        #log.debug('Class.on_subject_notify(%s, %s)' % (pspec, notifiers))
-        ElementItem.on_subject_notify(self, pspec,
-                ('namespace', 'namespace.name') + notifiers)
-        if self.subject:
-            self.on_subject_notify__namespace(self.subject)
-                                    
+    def on_named_element_name(self, event):
+        """
+        """
+        if self.subject and (event is None or self.subject is event.element):
+            self._name.text = self.subject.name
+            self.request_update()
 
-    def on_subject_notify__namespace(self, subject, pspec=None):
+
+    def on_named_element_namespace(self, event):
         """
         Add a line '(from ...)' to the class item if subject's namespace
         is not the same as the namespace of this diagram.
         """
         subject = self.subject
-        if subject and subject.namespace:
-            self._from.text = subject.namespace.name
-        else:
-            self._from.text = ''
-        self.request_update()
-
-
-    def on_subject_notify__namespace_name(self, subject, pspec=None):
-        """
-        Change the '(from ...)' line if the namespace's name changes.
-        """
-        self.on_subject_notify__namespace(subject, pspec)
+        if event is None or subject is event.element:
+            if subject and subject.namespace:
+                self._from.text = subject.namespace.name
+            else:
+                self._from.text = ''
+            self.request_update()
 
 
     def pre_update(self, context):
