@@ -23,12 +23,11 @@ from gaphor.event import TransactionBegin, TransactionCommit, TransactionRollbac
 from gaphor.transaction import TransactionError, transactional
 
 from gaphor.UML.event import ElementCreateEvent, ElementDeleteEvent, \
-                             FlushFactoryEvent, ModelFactoryEvent, \
-                             AttributeChangeEvent, AssociationSetEvent, \
+                             ModelFactoryEvent, AssociationSetEvent, \
                              AssociationAddEvent, AssociationDeleteEvent
-from gaphor.UML.interfaces import IElementCreateEvent, IElementDeleteEvent, \
-                                  IAttributeChangeEvent, IModelFactoryEvent, \
-                                  IAssociationChangeEvent
+from gaphor.UML.interfaces import IElementDeleteEvent, \
+                                  IAttributeChangeEvent, IModelFactoryEvent
+                                  
 
 from gaphor.action import action, build_action_group
 from gaphor.event import ActionExecuted
@@ -160,15 +159,13 @@ class UndoManager(object):
         """
         Add an action to undo. An action
         """
-        #log.debug('add_undo_action: %s %s' % (self._current_transaction, action))
-        if not self._current_transaction:
-            return
+        if self._current_transaction:
+            #log.debug('add_undo_action: %s %s' % (self._current_transaction, action))
+            self._current_transaction.add(action)
+            self._app.handle(UndoManagerStateChanged(self))
 
-        self._current_transaction.add(action)
-        component.handle(UndoManagerStateChanged(self))
-
-        # TODO: should this be placed here?
-        self._action_executed()
+            # TODO: should this be placed here?
+            self._action_executed()
 
     @component.adapter(TransactionCommit)
     def commit_transaction(self, event=None):
@@ -187,7 +184,7 @@ class UndoManager(object):
                 pass #log.debug('nothing to commit')
 
             self._current_transaction = None
-        component.handle(UndoManagerStateChanged(self))
+        self._app.handle(UndoManagerStateChanged(self))
         self._action_executed()
 
     @component.adapter(TransactionRollback)
@@ -212,7 +209,7 @@ class UndoManager(object):
                 # Discard all data collected in the rollback "transaction"
                 self.discard_transaction()
 
-            component.handle(UndoManagerStateChanged(self))
+            self._app.handle(UndoManagerStateChanged(self))
 
         self._action_executed()
 
@@ -223,7 +220,7 @@ class UndoManager(object):
         self._transaction_depth -= 1
         if self._transaction_depth == 0:
             self._current_transaction = None
-        component.handle(UndoManagerStateChanged(self))
+        self._app.handle(UndoManagerStateChanged(self))
         self._action_executed()
 
     @action(name='edit-undo', stock_id='gtk-undo', accel='<Control>z')
@@ -245,7 +242,7 @@ class UndoManager(object):
         self._redo_stack.append(self._current_transaction)
         self._current_transaction = None
         self._transaction_depth = 0
-        component.handle(UndoManagerStateChanged(self))
+        self._app.handle(UndoManagerStateChanged(self))
         self._action_executed()
 
     @action(name='edit-redo', stock_id='gtk-redo', accel='<Control>y')
@@ -265,7 +262,7 @@ class UndoManager(object):
         self._current_transaction = None
         self._transaction_depth = 0
 
-        component.handle(UndoManagerStateChanged(self))
+        self._app.handle(UndoManagerStateChanged(self))
         self._action_executed()
 
     def in_transaction(self):
@@ -287,7 +284,7 @@ class UndoManager(object):
     ## Undo Handlers
     ##
 
-    def _undo_handler(self, event):
+    def _gaphas_undo_handler(self, event):
         self.add_undo_action(lambda: state.saveapply(*event));
 
     def _register_undo_handlers(self):
@@ -302,7 +299,7 @@ class UndoManager(object):
         # Direct revert-statements from gaphas to the undomanager
         state.observers.add(state.revert_handler)
 
-        state.subscribers.add(self._undo_handler)
+        state.subscribers.add(self._gaphas_undo_handler)
 
     def _unregister_undo_handlers(self):
         self._app.unregister_handler(self.undo_create_event)
@@ -315,10 +312,10 @@ class UndoManager(object):
         from gaphas import state
         state.observers.discard(state.revert_handler)
 
-        state.subscribers.discard(self._undo_handler)
+        state.subscribers.discard(self._gaphas_undo_handler)
 
 
-    @component.adapter(IElementCreateEvent)
+    @component.adapter(ElementCreateEvent)
     def undo_create_event(self, event):
         factory = event.service
         # A factory is not always present, e.g. for DiagramItems
@@ -330,7 +327,7 @@ class UndoManager(object):
                 del factory._elements[element.id]
             except KeyError:
                 pass # Key was probably already removed in an unlink call
-            component.handle(ElementDeleteEvent(factory, element))
+            self._app.handle(ElementDeleteEvent(factory, element))
         self.add_undo_action(_undo_create_event)
 
 
@@ -344,7 +341,7 @@ class UndoManager(object):
         assert factory, 'No factory defined for %s (%s)' % (element, factory)
         def _undo_delete_event():
             factory._elements[element.id] = element
-            component.handle(ElementCreateEvent(factory, element))
+            self._app.handle(ElementCreateEvent(factory, element))
         self.add_undo_action(_undo_delete_event)
 
 
