@@ -16,14 +16,13 @@ from gaphas.geometry import distance_point_point, distance_point_point_fast, \
 from gaphas.item import Line
 from gaphas.tool import Tool, HandleTool, PlacementTool as _PlacementTool
 from gaphas.tool import ToolChain, HoverTool, ItemTool, RubberbandTool
-
-
+from gaphas.canvas import Context
 
 from gaphor.core import inject, Transaction, transactional
 
 from gaphor.diagram.interfaces import IEditor, IConnect
 
-__version__ = '$Revision: 2236 $'
+__version__ = '$Revision: 2325 $'
 
 
 class ConnectHandleTool(HandleTool):
@@ -293,6 +292,10 @@ class PlacementTool(_PlacementTool):
         self.after_handler = after_handler
         self._tx = None
 
+    @transactional
+    def create_item(self, view, x, y):
+        self._create_item(Context(view=view), x, y)
+
     def on_button_press(self, context, event):
         self._tx = Transaction()
         view = context.view
