@@ -11,9 +11,10 @@ import stock
 
 from zope import component
 
+from gaphor.application import Application
 from gaphor import UML
-from gaphor.UML.event import ModelFactoryEvent, FlushFactoryEvent
-from gaphor.UML.interfaces import IAttributeChangeEvent, IElementCreateEvent, IElementDeleteEvent
+from gaphor.UML.event import ElementCreateEvent, ModelFactoryEvent, FlushFactoryEvent
+from gaphor.UML.interfaces import IAttributeChangeEvent, IElementDeleteEvent
 from gaphor.UML.event import DerivedUnionSetEvent
 from gaphor.transaction import Transaction
 
@@ -22,7 +23,14 @@ from gaphor.transaction import Transaction
 # are UML.Namespace elements.
 _default_filter_list = (
     UML.Class,
+    UML.Interface,
     UML.Package,
+    UML.Component,
+    UML.Node,
+    UML.Artifact,
+    UML.Interaction,
+    UML.UseCase,
+    UML.Actor,
     UML.Diagram,
     UML.Profile,
     UML.Stereotype,
@@ -75,15 +83,28 @@ class NamespaceModel(gtk.GenericTreeModel):
 
         self.filter = _default_filter_list
 
-        component.provideHandler(self.flush)
-        component.provideHandler(self.refresh)
-        component.provideHandler(self._on_element_change)
-        component.provideHandler(self._on_element_create)
-        component.provideHandler(self._on_element_delete)
-        component.provideHandler(self._on_association_set)
+        Application.register_handler(self.flush)
+        Application.register_handler(self.refresh)
+        Application.register_handler(self._on_element_change)
+        Application.register_handler(self._on_element_create)
+        Application.register_handler(self._on_element_delete)
+        Application.register_handler(self._on_association_set)
 
         self._build_model()
 
+
+    def close(self):
+        """
+        Close the namespace model, unregister handlers.
+        """
+        Application.unregister_handler(self.flush)
+        Application.unregister_handler(self.refresh)
+        Application.unregister_handler(self._on_element_change)
+        Application.unregister_handler(self._on_element_create)
+        Application.unregister_handler(self._on_element_delete)
+        Application.unregister_handler(self._on_association_set)
+
+
     def path_from_element(self, e):
         if e:
             ns = e.namespace
@@ -95,6 +116,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         else:
             return ()
 
+
     def element_from_path(self, path):
         """
         Get the node form a path. None is returned if no node is found.
@@ -108,6 +130,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         except IndexError:
             return None
 
+
     @component.adapter(IAttributeChangeEvent)
     @catchall
     def _on_element_change(self, event):
@@ -148,10 +171,16 @@ class NamespaceModel(gtk.GenericTreeModel):
                                         [original] * len(parent_nodes),
                                         parent_nodes))
 
+
     def _add_elements(self, element):
         """
         Add a single element.
         """
+        if type(element) not in self.filter:
+            return
+        if element.namespace and type(element.namespace) not in self.filter:
+            return
+
         self._nodes.setdefault(element, [])
         parent = self._nodes[element.namespace]
         parent.append(element)
@@ -162,8 +191,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         # Add children
         if isinstance(element, UML.Namespace):
             for e in element.ownedMember:
-                if type(e) in self.filter:
-                    self._add_elements(e)
+                self._add_elements(e)
 
 
     def _remove_element(self, element):
@@ -181,12 +209,11 @@ class NamespaceModel(gtk.GenericTreeModel):
         remove(element)
 
 
-    @component.adapter(IElementCreateEvent)
+    @component.adapter(ElementCreateEvent)
     @catchall
     def _on_element_create(self, event):
         element = event.element
-        if event.service is self.factory and \
-                 type(element) in self.filter:
+        if event.service is self.factory:
             self._add_elements(element)
 
 
@@ -250,29 +277,33 @@ class NamespaceModel(gtk.GenericTreeModel):
                     parent.sort(key=_tree_sorter)
                     path = self.path_from_element(element)
                     self.row_inserted(path, self.get_iter(path))
-                elif type(element) in self.filter:
+                else:
                     self._add_elements(element)
             elif self._nodes.has_key(element):
                 # Non-existent: remove entirely
                 self._remove_element(element)
 
+
     @component.adapter(ModelFactoryEvent)
     def refresh(self, event=None):
         self.flush()
         self._build_model()
 
+
     @component.adapter(FlushFactoryEvent)
     def flush(self, event=None):
         for n in self._nodes[None]:
             self.row_deleted((0,))
         self._nodes = {None: []}
 
+
     def _build_model(self):
         toplevel = self.factory.select(lambda e: isinstance(e, UML.Namespace) and not e.namespace)
 
         for element in toplevel:
             self._add_elements(element)
 
+
     # TreeModel methods:
 
     def on_get_flags(self):
@@ -281,18 +312,21 @@ class NamespaceModel(gtk.GenericTreeModel):
         """
         return 0
 
+
     def on_get_n_columns(self):
         """
         Returns the number of columns in the model.
         """
         return 1
 
+
     def on_get_column_type(self, index):
         """
         Returns the type of a column in the model.
         """
         return gobject.TYPE_PYOBJECT
 
+
     def on_get_path(self, node):
         """
         Returns the path for a node as a tuple (0, 1, 1).
@@ -300,6 +334,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         path = self.path_from_element(node)
         return path
 
+
     def on_get_iter(self, path):
         """
         Returns the node corresponding to the given path.
@@ -308,6 +343,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         """
         return self.element_from_path(path)
 
+
     def on_get_value(self, node, column):
         """
         Returns the model element that matches 'node'.
@@ -315,6 +351,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         assert column == 0, 'column can only be 0'
         return node
 
+
     def on_iter_next(self, node):
         """
         Returns the next node at this level of the tree. None if no
@@ -324,8 +361,9 @@ class NamespaceModel(gtk.GenericTreeModel):
             parent = self._nodes[node.namespace]
             index = parent.index(node)
             return parent[index + 1]
-        except IndexError, e:
+        except (IndexError, ValueError), e:
             return None
+
         
     def on_iter_has_child(self, node):
         """
@@ -334,6 +372,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         n = self._nodes.get(node)
         return n or len(n) > 0
 
+
     def on_iter_children(self, node):
         """
         Returns the first child of this node, or None.
@@ -343,12 +382,14 @@ class NamespaceModel(gtk.GenericTreeModel):
         except KeyError:
             pass
 
+
     def on_iter_n_children(self, node):
         """
         Returns the number of children of this node.
         """
         return len(self._nodes[node])
 
+
     def on_iter_nth_child(self, node, n):
         """
         Returns the nth child of this node.
@@ -359,6 +400,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         except TypeError, e:
             return None
 
+
     def on_iter_parent(self, node):
         """
         Returns the parent of this node or None if no parent
@@ -434,6 +476,7 @@ class NamespaceView(gtk.TreeView):
         self.connect('drag-drop', NamespaceView.on_drag_drop)
         self.connect('drag-data-delete', NamespaceView.on_drag_data_delete)
 
+
     def get_selected_element(self):
         selection = self.get_selection()
         model, iter = selection.get_selected()
@@ -441,9 +484,11 @@ class NamespaceView(gtk.TreeView):
             return
         return model.get_value(iter, 0)
 
+
     def expand_root_nodes(self):
         self.expand_row((0,), False)
 
+
     def _set_pixbuf(self, column, cell, model, iter, data):
         value = model.get_value(iter, 0)
         try:
@@ -490,9 +535,11 @@ class NamespaceView(gtk.TreeView):
         except Exception, e:
             log.error('Could not create path from string "%s"' % path_str)
 
+
 #    def do_drag_begin (self, context):
 #        print 'do_drag_begin'
 
+
     def on_drag_data_get(self, context, selection_data, info, time):
         """
         Get the data to be dropped by on_drag_data_received().
@@ -508,12 +555,14 @@ class NamespaceView(gtk.TreeView):
             else:
                 selection_data.set(selection_data.target, 8, element.name)
 
+
     def on_drag_data_delete (self, context):
         """
         DnD magic. do not touch.
         """
         self.emit_stop_by_name('drag-data-delete')
 
+
     # Drop
     def on_drag_data_received(self, context, x, y, selection, info, time):
         """
@@ -568,6 +617,7 @@ class NamespaceView(gtk.TreeView):
                 selection = self.get_selection()
                 selection.select_path(path)
 
+
     def on_drag_drop(self, context, x, y, time):
         """
         DnD magic. do not touch
@@ -576,6 +626,7 @@ class NamespaceView(gtk.TreeView):
         self.drag_get_data(context, context.targets[-1], time)
         return 1
 
+
 gobject.type_register(NamespaceModel)
 gobject.type_register(NamespaceView)
 
