@@ -25,6 +25,13 @@ class Toolbox(gtk.VBox):
     3 connect to action
     """
 
+    TARGET_STRING = 0
+    TARGET_TOOLBOX_ACTION = 1
+    DND_TARGETS = [
+        ('STRING', 0, TARGET_STRING),
+        ('text/plain', 0, TARGET_STRING),
+        ('gaphor/toolbox-action', 0, TARGET_TOOLBOX_ACTION)]
+
     __gsignals__ = {
         'toggled': (gobject.SIGNAL_RUN_FIRST,
                     gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_INT))
@@ -32,6 +39,7 @@ class Toolbox(gtk.VBox):
 
     properties = inject('properties')
 
+
     def __init__(self, toolboxdef):
         """
         Create a new Toolbox instance. Wrapbox objects are generated
@@ -43,6 +51,7 @@ class Toolbox(gtk.VBox):
         self.buttons = []
         self._construct()
 
+
     def on_wrapbox_decorator_toggled(self, button, content):
         """
         This function is called when the Wrapbox decorator is clicked. It
@@ -63,6 +72,7 @@ class Toolbox(gtk.VBox):
         self.properties.set('ui.toolbox.%s' % button.toggle_id,
                             content.get_property('visible'))
 
+
     def make_wrapbox_decorator(self, title, content):
         """
         Create a gtk.VBox with in the top compartment a label that can be
@@ -102,6 +112,7 @@ class Toolbox(gtk.VBox):
 
         return vbox
 
+
     def toolbox_button(self, action_name, stock_id,
                        icon_size=gtk.ICON_SIZE_LARGE_TOOLBAR):
         button = gtk.ToggleButton()
@@ -113,17 +124,24 @@ class Toolbox(gtk.VBox):
         else:
             button.props.label = action_name
         button.action_name = action_name
+        
+        # Enable DND (behaviour like tree view)
+        button.drag_source_set(gtk.gdk.BUTTON1_MASK, self.DND_TARGETS,
+                gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK)
+        button.drag_source_set_icon_stock(stock_id)
+        button.connect('drag-data-get', self._button_drag_data_get)
+
         return button
 
+
     def _construct(self):
 
-        self.set_border_width(3)
+        #self.set_border_width(3)
 
         self.tooltips = gtk.Tooltips()
 
         for title, items in self.toolboxdef:
             wrapbox = Wrapbox()
-            action = None
             for action_name, label, stock_id in items:
                 button = self.toolbox_button(action_name, stock_id)
                 if label:
@@ -140,4 +158,9 @@ class Toolbox(gtk.VBox):
 
         self.tooltips.enable()
 
+
+    def _button_drag_data_get(self, button, context, selection_data, info, time):
+        selection_data.set(selection_data.target, 8, button.action_name)
+
+
 # vim:sw=4:et:ai
